@@ -13,7 +13,7 @@ async fn main() -> io::Result<()> {
     env_logger::init();
 
     Server::build()
-        .bind("echo", "127.0.0.1:8080", || {
+        .bind("echo", "127.0.0.1:8080", |_| {
             HttpService::build()
                 .client_timeout(Seconds(1))
                 .disconnect_timeout(Seconds(1))
@@ -30,7 +30,6 @@ async fn main() -> io::Result<()> {
                             .body(body),
                     )
                 })
-                .tcp()
         })?
         .run()
         .await
