@@ -2,19 +2,19 @@ use std::task::{Context, Poll};
 use std::time::{Duration, Instant};
 use std::{cell::RefCell, collections::VecDeque, future::Future, pin::Pin, rc::Rc};
 
-use h2::client::{Builder, Connection, SendRequest};
+use h2::client::{Builder, Connection as H2Connection, SendRequest};
 use http::uri::Authority;
+use ntex_tls::types::HttpProtocol;
 
 use crate::channel::pool;
-use crate::codec::{AsyncRead, AsyncWrite, ReadBuf};
-use crate::http::Protocol;
+use crate::io::IoBoxed;
 use crate::rt::spawn;
 use crate::service::Service;
 use crate::task::LocalWaker;
-use crate::time::{now, sleep, Millis, Sleep};
-use crate::util::{poll_fn, Bytes, HashMap};
+use crate::time::{now, Millis};
+use crate::util::{Bytes, HashMap};
 
-use super::connection::{ConnectionType, IoConnection};
+use super::connection::{Connection, ConnectionType};
 use super::error::ConnectError;
 use super::Connect;
 
@@ -29,18 +29,15 @@ impl From<Authority> for Key {
     }
 }
 
-type Waiter<Io> = pool::Sender<Result<IoConnection<Io>, ConnectError>>;
-type WaiterReceiver<Io> = pool::Receiver<Result<IoConnection<Io>, ConnectError>>;
+type Waiter = pool::Sender<Result<Connection, ConnectError>>;
+type WaiterReceiver = pool::Receiver<Result<Connection, ConnectError>>;
 
 /// Connections pool
-pub(super) struct ConnectionPool<T, Io: 'static>(Rc<T>, Rc<RefCell<Inner<Io>>>);
+pub(super) struct ConnectionPool<T>(Rc<T>, Rc<RefCell<Inner>>);
 
-impl<T, Io> ConnectionPool<T, Io>
+impl<T> ConnectionPool<T>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    T: Service<Request = Connect, Response = (Io, Protocol), Error = ConnectError>
-        + Unpin
-        + 'static,
+    T: Service<Connect, Response = IoBoxed, Error = ConnectError> + Unpin + 'static,
     T::Future: Unpin,
 {
     pub(super) fn new(
@@ -73,35 +70,26 @@ where
     }
 }
 
-impl<T, Io> Drop for ConnectionPool<T, Io>
-where
-    Io: 'static,
-{
+impl<T> Drop for ConnectionPool<T> {
     fn drop(&mut self) {
         self.1.borrow().waker.wake();
     }
 }
 
-impl<T, Io> Clone for ConnectionPool<T, Io>
-where
-    Io: 'static,
-{
+impl<T> Clone for ConnectionPool<T> {
     fn clone(&self) -> Self {
         ConnectionPool(self.0.clone(), self.1.clone())
     }
 }
 
-impl<T, Io> Service for ConnectionPool<T, Io>
+impl<T> Service<Connect> for ConnectionPool<T>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    T: Service<Request = Connect, Response = (Io, Protocol), Error = ConnectError>
-        + 'static,
+    T: Service<Connect, Response = IoBoxed, Error = ConnectError> + 'static,
     T::Future: Unpin,
 {
-    type Request = Connect;
-    type Response = IoConnection<Io>;
+    type Response = Connection;
     type Error = ConnectError;
-    type Future = Pin<Box<dyn Future<Output = Result<IoConnection<Io>, ConnectError>>>>;
+    type Future = Pin<Box<dyn Future<Output = Result<Connection, ConnectError>>>>;
 
     #[inline]
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
@@ -127,11 +115,12 @@ where
             };
 
             // acquire connection
-            match poll_fn(|cx| Poll::Ready(inner.borrow_mut().acquire(&key, cx))).await {
+            let result = inner.borrow_mut().acquire(&key);
+            match result {
                 // use existing connection
                 Acquire::Acquired(io, created) => {
                     trace!("Use existing connection for {:?}", req.uri);
-                    Ok(IoConnection::new(
+                    Ok(Connection::new(
                         io,
                         created,
                         Some(Acquired(key, Some(inner))),
@@ -144,7 +133,7 @@ where
                     OpenConnection::spawn(key, tx, inner, connector.call(req));
 
                     match rx.await {
-                        Err(_) => Err(ConnectError::Disconnected),
+                        Err(_) => Err(ConnectError::Disconnected(None)),
                         Ok(res) => res,
                     }
                 }
@@ -156,7 +145,7 @@ where
                     );
                     let rx = inner.borrow_mut().wait_for(req);
                     match rx.await {
-                        Err(_) => Err(ConnectError::Disconnected),
+                        Err(_) => Err(ConnectError::Disconnected(None)),
                         Ok(res) => res,
                     }
                 }
@@ -165,31 +154,31 @@ where
     }
 }
 
-enum Acquire<T> {
-    Acquired(ConnectionType<T>, Instant),
+enum Acquire {
+    Acquired(ConnectionType, Instant),
     Available,
     NotAvailable,
 }
 
-struct AvailableConnection<Io> {
-    io: ConnectionType<Io>,
+struct AvailableConnection {
+    io: ConnectionType,
     used: Instant,
     created: Instant,
 }
 
-pub(super) struct Inner<Io> {
+pub(super) struct Inner {
     conn_lifetime: Duration,
     conn_keep_alive: Duration,
     disconnect_timeout: Millis,
     limit: usize,
     acquired: usize,
-    available: HashMap<Key, VecDeque<AvailableConnection<Io>>>,
-    waiters: VecDeque<(Key, Connect, Waiter<Io>)>,
+    available: HashMap<Key, VecDeque<AvailableConnection>>,
+    waiters: VecDeque<(Key, Connect, Waiter)>,
     waker: LocalWaker,
-    pool: pool::Pool<Result<IoConnection<Io>, ConnectError>>,
+    pool: pool::Pool<Result<Connection, ConnectError>>,
 }
 
-impl<Io> Inner<Io> {
+impl Inner {
     fn reserve(&mut self) {
         self.acquired += 1;
     }
@@ -199,12 +188,9 @@ impl<Io> Inner<Io> {
     }
 }
 
-impl<Io> Inner<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+impl Inner {
     /// connection is not available, wait
-    fn wait_for(&mut self, connect: Connect) -> WaiterReceiver<Io> {
+    fn wait_for(&mut self, connect: Connect) -> WaiterReceiver {
         let (tx, rx) = self.pool.channel();
         let key: Key = connect.uri.authority().unwrap().clone().into();
         self.waiters.push_back((key, connect, tx));
@@ -226,7 +212,7 @@ where
         }
     }
 
-    fn acquire(&mut self, key: &Key, cx: &mut Context<'_>) -> Acquire<Io> {
+    fn acquire(&mut self, key: &Key) -> Acquire {
         self.cleanup();
 
         // check limits
@@ -246,22 +232,26 @@ where
                     || (now - conn.created) > self.conn_lifetime
                 {
                     if let ConnectionType::H1(io) = conn.io {
-                        CloseConnection::spawn(io, self.disconnect_timeout);
+                        spawn(async move {
+                            let _ = io.shutdown().await;
+                        });
                     }
                 } else {
-                    let mut io = conn.io;
-                    let mut buf = [0; 2];
-                    let mut read_buf = ReadBuf::new(&mut buf);
-                    if let ConnectionType::H1(ref mut s) = io {
-                        match Pin::new(s).poll_read(cx, &mut read_buf) {
-                            Poll::Pending => (),
-                            Poll::Ready(Ok(_)) if !read_buf.filled().is_empty() => {
-                                if let ConnectionType::H1(io) = io {
-                                    CloseConnection::spawn(io, self.disconnect_timeout);
-                                }
-                                continue;
+                    let io = conn.io;
+                    if let ConnectionType::H1(ref s) = io {
+                        if s.is_closed() {
+                            continue;
+                        }
+                        let is_valid = s.with_read_buf(|buf| {
+                            if buf.is_empty() || (buf.len() == 2 && &buf[..] == b"\r\n") {
+                                buf.clear();
+                                true
+                            } else {
+                                false
                             }
-                            _ => continue,
+                        });
+                        if !is_valid {
+                            continue;
                         }
                     }
                     return Acquire::Acquired(io, conn.created);
@@ -271,7 +261,7 @@ where
         Acquire::Available
     }
 
-    fn release_conn(&mut self, key: &Key, io: ConnectionType<Io>, created: Instant) {
+    fn release_conn(&mut self, key: &Key, io: ConnectionType, created: Instant) {
         self.acquired -= 1;
         self.available
             .entry(key.clone())
@@ -284,10 +274,12 @@ where
         self.check_availibility();
     }
 
-    fn release_close(&mut self, io: ConnectionType<Io>) {
+    fn release_close(&mut self, io: ConnectionType) {
         self.acquired -= 1;
         if let ConnectionType::H1(io) = io {
-            CloseConnection::spawn(io, self.disconnect_timeout);
+            spawn(async move {
+                let _ = io.shutdown().await;
+            });
         }
         self.check_availibility();
     }
@@ -300,19 +292,14 @@ where
     }
 }
 
-struct ConnectionPoolSupport<T, Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+struct ConnectionPoolSupport<T> {
     connector: T,
-    inner: Rc<RefCell<Inner<Io>>>,
+    inner: Rc<RefCell<Inner>>,
 }
 
-impl<T, Io> Future for ConnectionPoolSupport<T, Io>
+impl<T> Future for ConnectionPoolSupport<T>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    T: Service<Request = Connect, Response = (Io, Protocol), Error = ConnectError>
-        + Unpin,
+    T: Service<Connect, Response = IoBoxed, Error = ConnectError> + Unpin,
     T::Future: Unpin + 'static,
 {
     type Output = ();
@@ -337,11 +324,11 @@ where
             };
             let key = key.clone();
 
-            match inner.acquire(&key, cx) {
+            match inner.acquire(&key) {
                 Acquire::NotAvailable => break,
                 Acquire::Acquired(io, created) => {
                     let (key, _, tx) = inner.waiters.pop_front().unwrap();
-                    let _ = tx.send(Ok(IoConnection::new(
+                    let _ = tx.send(Ok(Connection::new(
                         io,
                         created,
                         Some(Acquired(key.clone(), Some(this.inner.clone()))),
@@ -363,96 +350,35 @@ where
     }
 }
 
-struct CloseConnection<T> {
-    io: T,
-    timeout: Option<Sleep>,
-    shutdown: bool,
-}
-
-impl<T> CloseConnection<T>
-where
-    T: AsyncWrite + AsyncRead + Unpin + 'static,
-{
-    fn spawn(io: T, timeout: Millis) {
-        spawn(Self {
-            io,
-            shutdown: false,
-            timeout: timeout.map(sleep),
-        });
-    }
-}
-
-impl<T> Future for CloseConnection<T>
-where
-    T: AsyncWrite + AsyncRead + Unpin,
-{
-    type Output = ();
-
-    fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<()> {
-        let mut this = self.as_mut();
-
-        // shutdown WRITE side
-        match Pin::new(&mut this.io).poll_shutdown(cx) {
-            Poll::Ready(Ok(())) => this.shutdown = true,
-            Poll::Pending => return Poll::Pending,
-            Poll::Ready(Err(_)) => return Poll::Ready(()),
-        }
-
-        // read until 0 or err
-        if let Some(ref timeout) = this.timeout {
-            match timeout.poll_elapsed(cx) {
-                Poll::Ready(_) => (),
-                Poll::Pending => {
-                    let mut buf = [0u8; 512];
-                    let mut read_buf = ReadBuf::new(&mut buf);
-                    loop {
-                        match Pin::new(&mut this.io).poll_read(cx, &mut read_buf) {
-                            Poll::Pending => return Poll::Pending,
-                            Poll::Ready(Err(_)) => return Poll::Ready(()),
-                            Poll::Ready(Ok(_)) => {
-                                if read_buf.filled().is_empty() {
-                                    return Poll::Ready(());
-                                }
-                                continue;
-                            }
-                        }
-                    }
-                }
-            }
-        }
-        Poll::Ready(())
-    }
-}
-
-struct OpenConnection<F, Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+struct OpenConnection<F> {
     fut: F,
     h2: Option<
         Pin<
             Box<
                 dyn Future<
                     Output = Result<
-                        (SendRequest<Bytes>, Connection<Io, Bytes>),
+                        (SendRequest<Bytes>, H2Connection<IoBoxed, Bytes>),
                         h2::Error,
                     >,
                 >,
             >,
         >,
     >,
-    tx: Option<Waiter<Io>>,
-    guard: Option<OpenGuard<Io>>,
+    tx: Option<Waiter>,
+    guard: Option<OpenGuard>,
+    disconnect_timeout: Millis,
 }
 
-impl<F, Io> OpenConnection<F, Io>
+impl<F> OpenConnection<F>
 where
-    F: Future<Output = Result<(Io, Protocol), ConnectError>> + Unpin + 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
+    F: Future<Output = Result<IoBoxed, ConnectError>> + Unpin + 'static,
 {
-    fn spawn(key: Key, tx: Waiter<Io>, inner: Rc<RefCell<Inner<Io>>>, fut: F) {
+    fn spawn(key: Key, tx: Waiter, inner: Rc<RefCell<Inner>>, fut: F) {
+        let disconnect_timeout = inner.borrow().disconnect_timeout;
+
         spawn(OpenConnection {
             fut,
+            disconnect_timeout,
             h2: None,
             tx: Some(tx),
             guard: Some(OpenGuard {
@@ -463,10 +389,9 @@ where
     }
 }
 
-impl<F, Io> Future for OpenConnection<F, Io>
+impl<F> Future for OpenConnection<F>
 where
-    F: Future<Output = Result<(Io, Protocol), ConnectError>> + Unpin,
-    Io: AsyncRead + AsyncWrite + Unpin,
+    F: Future<Output = Result<IoBoxed, ConnectError>> + Unpin,
 {
     type Output = ();
 
@@ -478,7 +403,7 @@ where
             return match Pin::new(h2).poll(cx) {
                 Poll::Ready(Ok((snd, connection))) => {
                     // h2 connection is ready
-                    let conn = IoConnection::new(
+                    let conn = Connection::new(
                         ConnectionType::H2(snd),
                         now(),
                         Some(this.guard.take().unwrap().consume()),
@@ -511,11 +436,18 @@ where
                 }
                 Poll::Ready(())
             }
-            Poll::Ready(Ok((io, proto))) => {
-                trace!("Connection is established");
-                // handle http1 proto
-                if proto == Protocol::Http1 {
-                    let conn = IoConnection::new(
+            Poll::Ready(Ok(io)) => {
+                io.set_disconnect_timeout(this.disconnect_timeout);
+
+                // handle http2 proto
+                if io.query::<HttpProtocol>().get() == Some(HttpProtocol::Http2) {
+                    log::trace!("Connection is established, start http2 handshake");
+                    // init http2 handshake
+                    this.h2 = Some(Box::pin(Builder::new().handshake(io)));
+                    self.poll(cx)
+                } else {
+                    log::trace!("Connection is established, init http1 connection");
+                    let conn = Connection::new(
                         ConnectionType::H1(io),
                         now(),
                         Some(this.guard.take().unwrap().consume()),
@@ -525,10 +457,6 @@ where
                         conn.release()
                     }
                     Poll::Ready(())
-                } else {
-                    // init http2 handshake
-                    this.h2 = Some(Box::pin(Builder::new().handshake(io)));
-                    self.poll(cx)
                 }
             }
             Poll::Pending => Poll::Pending,
@@ -536,27 +464,18 @@ where
     }
 }
 
-struct OpenGuard<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+struct OpenGuard {
     key: Key,
-    inner: Option<Rc<RefCell<Inner<Io>>>>,
+    inner: Option<Rc<RefCell<Inner>>>,
 }
 
-impl<Io> OpenGuard<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
-    fn consume(mut self) -> Acquired<Io> {
+impl OpenGuard {
+    fn consume(mut self) -> Acquired {
         Acquired(self.key.clone(), self.inner.take())
     }
 }
 
-impl<Io> Drop for OpenGuard<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+impl Drop for OpenGuard {
     fn drop(&mut self) {
         if let Some(i) = self.inner.take() {
             let mut inner = i.as_ref().borrow_mut();
@@ -566,20 +485,17 @@ where
     }
 }
 
-pub(super) struct Acquired<T>(Key, Option<Rc<RefCell<Inner<T>>>>);
+pub(super) struct Acquired(Key, Option<Rc<RefCell<Inner>>>);
 
-impl<T> Acquired<T>
-where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-{
-    pub(super) fn close(&mut self, conn: IoConnection<T>) {
+impl Acquired {
+    pub(super) fn close(&mut self, conn: Connection) {
         if let Some(inner) = self.1.take() {
             let (io, _) = conn.into_inner();
             inner.as_ref().borrow_mut().release_close(io);
         }
     }
 
-    pub(super) fn release(&mut self, conn: IoConnection<T>) {
+    pub(super) fn release(&mut self, conn: Connection) {
         if let Some(inner) = self.1.take() {
             let (io, created) = conn.into_inner();
             inner
@@ -590,7 +506,7 @@ where
     }
 }
 
-impl<T> Drop for Acquired<T> {
+impl Drop for Acquired {
     fn drop(&mut self) {
         if let Some(inner) = self.1.take() {
             inner.borrow_mut().release();
@@ -604,8 +520,7 @@ mod tests {
 
     use super::*;
     use crate::{
-        http::client::Connection, http::Uri, service::fn_service, testing::Io,
-        util::lazy,
+        http::Uri, io as nio, service::fn_service, testing::Io, time::sleep, util::lazy,
     };
 
     #[crate::rt_test]
@@ -617,7 +532,7 @@ mod tests {
             fn_service(move |req| {
                 let (client, server) = Io::create();
                 store2.borrow_mut().push((req, server));
-                Box::pin(async move { Ok((client, Protocol::Http1)) })
+                Box::pin(async move { Ok(IoBoxed::from(nio::Io::new(client))) })
             }),
             Duration::from_secs(10),
             Duration::from_secs(10),
@@ -644,7 +559,7 @@ mod tests {
         let conn = pool.call(req.clone()).await.unwrap();
         assert_eq!(store.borrow().len(), 1);
         assert!(format!("{:?}", conn).contains("H1Connection"));
-        assert_eq!(conn.protocol(), Protocol::Http1);
+        assert_eq!(conn.protocol(), HttpProtocol::Http1);
         assert_eq!(pool.1.borrow().acquired, 1);
 
         // pool is full, waiting
