@@ -1,38 +1,30 @@
 //! General purpose tcp server
-#![allow(clippy::type_complexity)]
-use std::sync::atomic::{AtomicUsize, Ordering};
-use std::task::{Context, Poll};
-use std::{future::Future, io, pin::Pin};
+use std::{future::Future, io, pin::Pin, task::Context, task::Poll};
 
 use async_channel::Sender;
 use async_oneshot as oneshot;
 
-use crate::util::counter::Counter;
-
 mod accept;
 mod builder;
 mod config;
 mod service;
-mod signals;
 mod socket;
 mod test;
 mod worker;
 
 #[cfg(feature = "openssl")]
-pub mod openssl;
+pub use ntex_tls::openssl;
 
 #[cfg(feature = "rustls")]
-pub mod rustls;
+pub use ntex_tls::rustls;
+
+pub use ntex_tls::max_concurrent_ssl_accept;
 
 pub(crate) use self::builder::create_tcp_listener;
 pub use self::builder::ServerBuilder;
-pub use self::config::{ServiceConfig, ServiceRuntime};
-pub use self::service::StreamServiceFactory;
+pub use self::config::{Config, ServiceConfig, ServiceRuntime};
 pub use self::test::{build_test_server, test_server, TestServer};
 
-#[doc(hidden)]
-pub use self::socket::FromStream;
-
 #[non_exhaustive]
 #[derive(Copy, Clone, Debug, PartialEq, Eq)]
 /// Server readiness status
@@ -59,24 +51,6 @@ pub fn build() -> ServerBuilder {
     ServerBuilder::default()
 }
 
-/// Sets the maximum per-worker concurrent ssl connection establish process.
-///
-/// All listeners will stop accepting connections when this limit is
-/// reached. It can be used to limit the global SSL CPU usage.
-///
-/// By default max connections is set to a 256.
-pub fn max_concurrent_ssl_accept(num: usize) {
-    MAX_SSL_ACCEPT.store(num, Ordering::Relaxed);
-}
-
-#[allow(dead_code)]
-pub(self) const ZERO: std::time::Duration = std::time::Duration::from_millis(0);
-pub(self) static MAX_SSL_ACCEPT: AtomicUsize = AtomicUsize::new(256);
-
-thread_local! {
-    static MAX_SSL_ACCEPT_COUNTER: Counter = Counter::new(MAX_SSL_ACCEPT.load(Ordering::Relaxed));
-}
-
 /// Ssl error combinded with service error.
 #[derive(Debug)]
 pub enum SslError<E> {
@@ -89,7 +63,7 @@ enum ServerCommand {
     WorkerFaulted(usize),
     Pause(oneshot::Sender<()>),
     Resume(oneshot::Sender<()>),
-    Signal(signals::Signal),
+    Signal(crate::rt::Signal),
     /// Whether to try and shut down gracefully
     Stop {
         graceful: bool,
@@ -113,7 +87,7 @@ impl Server {
         ServerBuilder::default()
     }
 
-    fn signal(&self, sig: signals::Signal) {
+    fn signal(&self, sig: crate::rt::Signal) {
         let _ = self.0.try_send(ServerCommand::Signal(sig));
     }
 
