@@ -3,8 +3,9 @@ use std::{io, net, sync::mpsc, thread};
 
 use socket2::{Domain, SockAddr, Socket, Type};
 
-use crate::rt::{net::TcpStream, System};
-use crate::server::{Server, ServerBuilder, StreamServiceFactory};
+use crate::rt::{tcp_connect, System};
+use crate::server::{Server, ServerBuilder};
+use crate::{io::Io, service::ServiceFactory};
 
 /// Start test server
 ///
@@ -37,21 +38,25 @@ use crate::server::{Server, ServerBuilder, StreamServiceFactory};
 ///     assert!(response.status().is_success());
 /// }
 /// ```
-pub fn test_server<F: StreamServiceFactory<TcpStream>>(factory: F) -> TestServer {
+pub fn test_server<F, R>(factory: F) -> TestServer
+where
+    F: Fn() -> R + Send + Clone + 'static,
+    R: ServiceFactory<Io>,
+{
     let (tx, rx) = mpsc::channel();
 
     // run server in separate thread
     thread::spawn(move || {
-        let mut sys = System::new("ntex-test-server");
+        let sys = System::new("ntex-test-server");
         let tcp = net::TcpListener::bind("127.0.0.1:0").unwrap();
         let local_addr = tcp.local_addr().unwrap();
 
         sys.exec(|| {
             Server::build()
-                .listen("test", tcp, factory)?
+                .listen("test", tcp, move |_| factory())?
                 .workers(1)
                 .disable_signals()
-                .start();
+                .run();
             Ok::<_, io::Error>(())
         })?;
 
@@ -73,13 +78,10 @@ where
 
     // run server in separate thread
     thread::spawn(move || {
-        let mut sys = System::new("ntex-test-server");
+        let sys = System::new("ntex-test-server");
 
         sys.exec(|| {
-            factory(Server::build())
-                .workers(1)
-                .disable_signals()
-                .start();
+            factory(Server::build()).workers(1).disable_signals().run();
         });
 
         tx.send(System::current()).unwrap();
@@ -105,9 +107,9 @@ impl TestServer {
         self.addr
     }
 
-    /// Connect to server, return TcpStream
-    pub async fn connect(&self) -> io::Result<TcpStream> {
-        TcpStream::connect(self.addr).await
+    /// Connect to server, return Io
+    pub async fn connect(&self) -> io::Result<Io> {
+        tcp_connect(self.addr).await
     }
 
     /// Stop http server
